package com.seeyon.apps.govdoc.helper;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.seeyon.apps.agent.bo.AgentDetailModel;
import com.seeyon.apps.agent.bo.AgentModel;
import com.seeyon.apps.agent.bo.MemberAgentBean;
import com.seeyon.apps.doc.bo.DocResourceBO;
import com.seeyon.apps.govdoc.po.EdocLeaderPishiNo;
import com.seeyon.apps.govdoc.util.GovdocParamUtil;
import com.seeyon.apps.govdoc.vo.GovdocListVO;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.enums.AffairExtPropEnums;
import com.seeyon.ctp.common.affair.util.AffairUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.config.ConfigItem;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumBean;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem;
import com.seeyon.ctp.common.po.template.CtpTemplate;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.util.Datetimes;
import com.seeyon.ctp.util.ParamUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.v3x.common.taglibs.functions.Functions;
import com.seeyon.v3x.edoc.domain.EdocElement;
import com.seeyon.v3x.worktimeset.exception.WorkTimeSetExecption;

/**
 * 公文列表辅助类
 */
public class GovdocListHelper extends GovdocHelper {
	
	private static final String  PROXYSTR= ResourceUtil.getString("collaboration.proxy");
	
	/**
	 * 用于公文查询设置自定义条件
	 * @return
	 * @throws BusinessException
	 */
	public static List<String> getQueryConditionList() throws BusinessException {
		List<String> list = new ArrayList<String>();
		list.add("subject");
		list.add("serial_no");
		list.add("doc_mark");
		list.add("send_type");
		list.add("doc_type");
		list.add("create_person");
		list.add("createdate");
		list.add("send_unit");
		list.add("send_to");
		list.add("send_department");
		list.add("issuer");
		list.add("signing_date");
		list.add("send_department");
		list.add("hasArchive");
		list.add("archiveName");
		list.add("copy_to");
		list.add("report_to");
		list.add("receive_unit");
		list.add("baopiyijian");
		list.add("feedback");
		return list;
	}
	
	/**
	 * 公文自定义查询时设置列表数据
	 * @param name
	 * @param vo
	 * @return
	 */
	public static String getListColumnValue(String name, GovdocListVO vo) {
		String value = ""; 
		if("subject".equals(name)) {
			value = vo.getSubject();
		} else if("docMark".equals(name)) {
			value = vo.getDocMark();
		}else if("registerDate".equals(name)) {
			if(vo.getCreateTime() != null) {
				value = Datetimes.format(vo.getCreateTime(), "yyy-MM-dd HH:mm");
			}
		}else if("serialNo".equals(name)) {
			value = vo.getSerialNo();
		} else if("sendunit".equals(name.toLowerCase())) {
			value = vo.getSendUnit();
		} else if("sendDepartment".equals(name)) {
			value = vo.getSendDepartment();
		} else if("startTime".equals(name)) {
			if(vo.getStartTime() != null) {
				value = Datetimes.format(vo.getStartTime(), "yyy-MM-dd");
			}
		} else if("currentNodesInfo".equals(name)) {
			value = vo.getCurrentNodesInfo();
		} else if("sendTo".equals(name)) {
			value = vo.getSendTo();
		} else if("copyTo".equals(name)) {
			value = vo.getCopyTo();
		} else if("doctype".equals(name.toLowerCase())) {
			value = vo.getDocType();
		} else if("sendtype".equals(name.toLowerCase())) {
			value = vo.getSendType();
		} else if("unitlevel".equals(name.toLowerCase())) {
			value = vo.getUnitLevel();
		} else if("secretlevel".equals(name.toLowerCase())) {
			value = vo.getSecretLevel();
		} else if("keepperiodtxt".equals(name.toLowerCase())) {
			value = vo.getKeepPeriodTxt();
		} else if("urgentlevel".equals(name.toLowerCase())) {
			value = vo.getUrgentLevel();
		} else if("issuer".equals(name)) {
			value = vo.getIssuer();
		} else if("signingDate".equals(name)) {
			if(vo.getSigningDate() != null) {
				value = Datetimes.format(vo.getSigningDate(), "yyy-MM-dd");
			}
		} else if("packTime".equals(name)) {
			if(vo.getPackTime() != null) {
				value = Datetimes.format(vo.getPackTime(), "yyy-MM-dd");
			}
		} else if("createPerson".equals(name)) {
			value = vo.getCreatePerson();
		} else if("transferStatus".equals(name) || "summaryTransferStatusName".equals(name)) {
			value = vo.getSummaryTransferStatusName();
		} else if("review".equals(name)) {
			value = vo.getReview();
		} else if("copies".equals(name)) {
			if(vo.getCopies() != null) {
				value = vo.getCopies() + "";
			}
		} else if("hasArchive".equals(name)) {//是否归档
			if(vo.getHasArchive() != null) {
				if(vo.getHasArchive()) {
	    			vo.setHasArchiveTxt(ResourceUtil.getString("govdoc.canArchive.label.yes"));
	    		} else {
	    			vo.setHasArchiveTxt(ResourceUtil.getString("govdoc.canArchive.label.no"));
	    		}
				value = vo.getHasArchiveTxt();
			}
		} else if("archiveName".equals(name)) {
			if(Strings.isNotBlank(vo.getArchiveName())) {
				value = vo.getArchiveName();
			}
		} else if("createTime".equals(name)) {//添加字段值判断，回写
			if(vo.getCreateTime() != null) {
				value = Datetimes.format(vo.getCreateTime(), "yyy-MM-dd");
			}
		} else if("receiptDate".equals(name)) {
			if(vo.getReceiptDate() != null) {
				value = Datetimes.format(vo.getReceiptDate(), "yyy-MM-dd");
			}
		} else if("registrationDate".equals(name)) {
			if(vo.getRegistrationDate() != null) {
				value = Datetimes.format(vo.getRegistrationDate(), "yyy-MM-dd");
			}
		} else if("undertaker".equals(name)) {
			value=vo.getUndertaker();
		} else if("signPerson".equals(name)) {
			value=vo.getSignPerson();
		} else if("signMark".equals(name)) {
			value=vo.getSignMark();
		} else if("leaderCommondNo".equals(name)) {
			value = vo.getLeaderCommondNo();
		} else if("keepPeriod".equals(name)) {
			value = vo.getKeepPeriodTxt();
		} else if("undertakenoffice".equals(name)) {
			value = vo.getUndertakenoffice();
		}
		//自定义元素
		else if("string1".equals(name)) {
			value = vo.getString1();
		}
		else if("string2".equals(name)) {
			value = vo.getString2();
		}
		else if("string3".equals(name)) {
			value = vo.getString3();
		}
		else if("string4".equals(name)) {
			value = vo.getString4();
		}
		else if("string5".equals(name)) {
			value = vo.getString5();
		}
		else if("string6".equals(name)) {
			value = vo.getString6();
		}
		else if("string7".equals(name)) {
			value = vo.getString7();
		}
		else if("string8".equals(name)) {
			value = vo.getString8();
		}
		else if("string9".equals(name)) {
			value = vo.getString9();
		}
		else if("string10".equals(name)) {
			value = vo.getString10();
		}
		else if("string11".equals(name)) {
			value = vo.getString11();
		}
		else if("string12".equals(name)) {
			value = vo.getString12();
		}
		else if("string13".equals(name)) {
			value = vo.getString13();
		}
		else if("string14".equals(name)) {
			value = vo.getString14();
		}
		else if("string15".equals(name)) {
			value = vo.getString15();
		}
		else if("string16".equals(name)) {
			value = vo.getString16();
		}
		else if("string17".equals(name)) {
			value = vo.getString17();
		}
		else if("string18".equals(name)) {
			value = vo.getString18();
		}
		else if("string19".equals(name)) {
			value = vo.getString19();
		}
		else if("string20".equals(name)) {
			value = vo.getString20();
		}
		else if("string21".equals(name)) {
			value = vo.getString21();
		}
		else if("string22".equals(name)) {
			value = vo.getString22();
		}
		else if("string23".equals(name)) {
			value = vo.getString23();
		}
		else if("string24".equals(name)) {
			value = vo.getString24();
		}
		else if("string25".equals(name)) {
			value = vo.getString25();
		}
		else if("string26".equals(name)) {
			value = vo.getString26();
		}
		else if("string27".equals(name)) {
			value = vo.getString27();
		}
		else if("string28".equals(name)) {
			value = vo.getString28();
		}
		else if("string29".equals(name)) {
			value = vo.getString29();
		}
		else if("string30".equals(name)) {
			value = vo.getString30();
		}
		else if("integer1".equals(name)) {
			value = vo.getInteger1().toString();
		}
		else if("integer2".equals(name)) {
			value = vo.getInteger2().toString();
		}
		else if("integer3".equals(name)) {
			value = vo.getInteger3().toString();
		}
		else if("integer4".equals(name)) {
			value = vo.getInteger4().toString();
		}
		else if("integer5".equals(name)) {
			value = vo.getInteger5().toString();
		}
		else if("integer6".equals(name)) {
			value = vo.getInteger6().toString();
		}
		else if("integer7".equals(name)) {
			value = vo.getInteger7().toString();
		}
		else if("integer8".equals(name)) {
			value = vo.getInteger8().toString();
		}
		else if("integer9".equals(name)) {
			value = vo.getInteger9().toString();
		}
		else if("integer10".equals(name)) {
			value = vo.getInteger10().toString();
		}
		else if("integer11".equals(name)) {
			value = vo.getInteger11().toString();
		}
		else if("integer12".equals(name)) {
			value = vo.getInteger12().toString();
		}
		else if("integer13".equals(name)) {
			value = vo.getInteger13().toString();
		}
		else if("integer14".equals(name)) {
			value = vo.getInteger14().toString();
		}
		else if("integer15".equals(name)) {
			value = vo.getInteger15().toString();
		}
		else if("integer16".equals(name)) {
			value = vo.getInteger16().toString();
		}
		else if("integer17".equals(name)) {
			value = vo.getInteger17().toString();
		}
		else if("integer18".equals(name)) {
			value = vo.getInteger18().toString();
		}
		else if("integer19".equals(name)) {
			value = vo.getInteger19().toString();
		}
		else if("integer20".equals(name)) {
			value = vo.getInteger20().toString();
		}
		else if("decimal1".equals(name)) {
			value = vo.getDecimal1().toString();
		}
		else if("decimal2".equals(name)) {
			value = vo.getDecimal2().toString();
		}
		else if("decimal3".equals(name)) {
			value = vo.getDecimal3().toString();
		}
		else if("decimal4".equals(name)) {
			value = vo.getDecimal4().toString();
		}
		else if("decimal5".equals(name)) {
			value = vo.getDecimal5().toString();
		}
		else if("decimal6".equals(name)) {
			value = vo.getDecimal6().toString();
		}
		else if("decimal7".equals(name)) {
			value = vo.getDecimal7().toString();
		}
		else if("decimal8".equals(name)) {
			value = vo.getDecimal8().toString();
		}
		else if("decimal9".equals(name)) {
			value = vo.getDecimal9().toString();
		}
		else if("decimal10".equals(name)) {
			value = vo.getDecimal10().toString();
		}
		else if("decimal11".equals(name)) {
			value = vo.getDecimal11().toString();
		}
		else if("decimal12".equals(name)) {
			value = vo.getDecimal12().toString();
		}
		else if("decimal13".equals(name)) {
			value = vo.getDecimal13().toString();
		}
		else if("decimal14".equals(name)) {
			value = vo.getDecimal14().toString();
		}
		else if("decimal15".equals(name)) {
			value = vo.getDecimal15().toString();
		}
		else if("decimal16".equals(name)) {
			value = vo.getDecimal16().toString();
		}
		else if("decimal17".equals(name)) {
			value = vo.getDecimal17().toString();
		}
		else if("decimal18".equals(name)) {
			value = vo.getDecimal18().toString();
		}
		else if("decimal19".equals(name)) {
			value = vo.getDecimal19().toString();
		}
		else if("decimal20".equals(name)) {
			value = vo.getDecimal20().toString();
		}
		else if("date1".equals(name)) {
			if(vo.getDate1() != null) {
				value = Datetimes.format(vo.getDate1(), "yyy-MM-dd");
			}
		}
		else if("date2".equals(name)) {
			if(vo.getDate2() != null) {
				value = Datetimes.format(vo.getDate2(), "yyy-MM-dd");
			}
		}
		else if("date3".equals(name)) {
			if(vo.getDate3() != null) {
				value = Datetimes.format(vo.getDate3(), "yyy-MM-dd");
			}
		}
		else if("date4".equals(name)) {
			if(vo.getDate4() != null) {
				value = Datetimes.format(vo.getDate4(), "yyy-MM-dd");
			}
		}
		else if("date5".equals(name)) {
			if(vo.getDate5() != null) {
				value = Datetimes.format(vo.getDate5(), "yyy-MM-dd");
			}
		}
		else if("date6".equals(name)) {
			if(vo.getDate6() != null) {
				value = Datetimes.format(vo.getDate6(), "yyy-MM-dd");
			}
		}
		else if("date7".equals(name)) {
			if(vo.getDate7() != null) {
				value = Datetimes.format(vo.getDate7(), "yyy-MM-dd");
			}
		}
		else if("date8".equals(name)) {
			if(vo.getDate8() != null) {
				value = Datetimes.format(vo.getDate8(), "yyy-MM-dd");
			}
		}
		else if("date9".equals(name)) {
			if(vo.getDate9() != null) {
				value = Datetimes.format(vo.getDate9(), "yyy-MM-dd");
			}
		}
		else if("date10".equals(name)) {
			if(vo.getDate10() != null) {
				value = Datetimes.format(vo.getDate10(), "yyy-MM-dd");
			}
		}
		else if("date11".equals(name)) {
			if(vo.getDate11() != null) {
				value = Datetimes.format(vo.getDate11(), "yyy-MM-dd");
			}
		}
		else if("date12".equals(name)) {
			if(vo.getDate12() != null) {
				value = Datetimes.format(vo.getDate12(), "yyy-MM-dd");
			}
		}
		else if("date13".equals(name)) {
			if(vo.getDate13() != null) {
				value = Datetimes.format(vo.getDate13(), "yyy-MM-dd");
			}
		}
		else if("date14".equals(name)) {
			if(vo.getDate14() != null) {
				value = Datetimes.format(vo.getDate14(), "yyy-MM-dd");
			}
		}
		else if("date15".equals(name)) {
			if(vo.getDate15() != null) {
				value = Datetimes.format(vo.getDate15(), "yyy-MM-dd");
			}
		}
		else if("date16".equals(name)) {
			if(vo.getDate16() != null) {
				value = Datetimes.format(vo.getDate16(), "yyy-MM-dd");
			}
		}
		else if("date17".equals(name)) {
			if(vo.getDate17() != null) {
				value = Datetimes.format(vo.getDate17(), "yyy-MM-dd");
			}
		}
		else if("date18".equals(name)) {
			if(vo.getDate18() != null) {
				value = Datetimes.format(vo.getDate18(), "yyy-MM-dd");
			}
		}
		else if("date18".equals(name)) {
			if(vo.getDate18() != null) {
				value = Datetimes.format(vo.getDate18(), "yyy-MM-dd");
			}
		}
		else if("date20".equals(name)) {
			if(vo.getDate20() != null) {
				value = Datetimes.format(vo.getDate20(), "yyy-MM-dd");
			}
		}
		else if("list1".equals(name)) {
			value = vo.getList1();
		}
		else if("list2".equals(name)) {
			value = vo.getList2();
		}
		else if("list3".equals(name)) {
			value = vo.getList3();
		}
		else if("list4".equals(name)) {
			value = vo.getList4();
		}
		else if("list5".equals(name)) {
			value = vo.getList5();
		}
		else if("list6".equals(name)) {
			value = vo.getList6();
		}
		else if("list7".equals(name)) {
			value = vo.getList7();
		}
		else if("list8".equals(name)) {
			value = vo.getList8();
		}
		else if("list9".equals(name)) {
			value = vo.getList9();
		}
		else if("list10".equals(name)) {
			value = vo.getList10();
		}
		else if("list11".equals(name)) {
			value = vo.getList11();
		}
		else if("list12".equals(name)) {
			value = vo.getList12();
		}
		else if("list13".equals(name)) {
			value = vo.getList13();
		}
		else if("list14".equals(name)) {
			value = vo.getList14();
		}
		else if("list15".equals(name)) {
			value = vo.getList15();
		}
		else if("list16".equals(name)) {
			value = vo.getList16();
		}
		else if("list17".equals(name)) {
			value = vo.getList17();
		}
		else if("list18".equals(name)) {
			value = vo.getList18();
		}
		else if("list19".equals(name)) {
			value = vo.getList19();
		}
		else if("list20".equals(name)) {
			value = vo.getList20();
		}
		else if("text1".equals(name)) {
			value = vo.getText1();
		}
		else if("text2".equals(name)) {
			value = vo.getText2();
		}
		else if("text3".equals(name)) {
			value = vo.getText3();
		}
		else if("text4".equals(name)) {
			value = vo.getText4();
		}
		else if("text5".equals(name)) {
			value = vo.getText5();
		}
		else if("text6".equals(name)) {
			value = vo.getText6();
		}
		else if("text7".equals(name)) {
			value = vo.getText7();
		}
		else if("text8".equals(name)) {
			value = vo.getText8();
		}
		else if("text9".equals(name)) {
			value = vo.getText9();
		}
		else if("text10".equals(name)) {
			value = vo.getText10();
		}
		else if("text11".equals(name)) {
			value = vo.getText11();
		}
		else if("text12".equals(name)) {
			value = vo.getText12();
		}
		else if("text13".equals(name)) {
			value = vo.getText13();
		}
		else if("text14".equals(name)) {
			value = vo.getText14();
		}
		else if("text15".equals(name)) {
			value = vo.getText15();
		}
		return value;
	}

	/**
	 * 公文自定义查询时，从前端获取查询元素
	 * @param name
	 * @param vo
	 * @return
	 */
	public static List<Map<String, String>> getQueryListColumn(Map<String, String> params) {
		List<Map<String, String>> columns = new ArrayList<Map<String, String>>();
		
		if(Strings.isNotBlank(params.get("customColumns"))) {
			String[] columnArr = params.get("customColumns").split(",");
			String[] displayArr = params.get("customDisplayColumns").split(",");
			Map<String, String> paramMap = null;
			for(int i = 0; i<columnArr.length; i++) {
				paramMap = new HashMap<String, String>();
				paramMap.put("name", columnArr[i]);
				paramMap.put("display", displayArr[i]);
				columns.add(paramMap);
			}
		} else {
			String govdocType = ParamUtil.getString(params, "govdocType", "");
			//文件密级
			Map<String, String> paramMap = new HashMap<String, String>();
			paramMap.put("name", "secretLevel");
			paramMap.put("display", ResourceUtil.getString("govdoc.secretLevel.label"));
			columns.add(paramMap);
			//紧急程度
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "urgentLevel");
			paramMap.put("display", ResourceUtil.getString("govdoc.urgentLevel.label"));
			columns.add(paramMap);
			//保密期限
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "keepPeriodTxt");
			paramMap.put("display", ResourceUtil.getString("govdoc.keepPeriod.label"));
			columns.add(paramMap);
			//公文级别
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "unitLevel");
			paramMap.put("display", ResourceUtil.getString("govdoc.unitLevel.label"));
			columns.add(paramMap);
			//标题
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "subject");
			paramMap.put("display", ResourceUtil.getString("common.subject.label"));
			columns.add(paramMap);
			//文号
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "docMark");
			paramMap.put("display", ResourceUtil.getString("govdoc.docMark.label"));
			columns.add(paramMap);
			if("1".equals(govdocType) || "3".equals(govdocType)) {//发文、签报
				//主送单位
				paramMap = new HashMap<String, String>();
				paramMap.put("name", "sendTo");
				paramMap.put("display", ResourceUtil.getString("edoc.element.sendtounit"));
				columns.add(paramMap);
				//建文人
				paramMap = new HashMap<String, String>();
				paramMap.put("name", "issuer");
				paramMap.put("display", ResourceUtil.getString("edoc.element.issuer"));
				columns.add(paramMap);
				//发文时间
				paramMap = new HashMap<String, String>();
				paramMap.put("name", "signingDate");
				paramMap.put("display", ResourceUtil.getString("edoc.element.sendingdate"));
				columns.add(paramMap);	
			} else if("2,4".equals(govdocType)) {//收文
				//来文单位
				paramMap = new HashMap<String, String>();
				paramMap.put("name", "sendUnit");
				paramMap.put("display", ResourceUtil.getString("edoc.edoctitle.fromUnit.label"));
				columns.add(paramMap);
				//登记人
				paramMap = new HashMap<String, String>();
				paramMap.put("name", "createPerson");
				paramMap.put("display", ResourceUtil.getString("edoc.edoctitle.regPerson.label"));
				columns.add(paramMap);
				//登记日期
				paramMap = new HashMap<String, String>();
				paramMap.put("name", "registerDate");
				paramMap.put("display", ResourceUtil.getString("edoc.edoctitle.regDate.label"));
				columns.add(paramMap);
			}
			//拟稿人
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "createPerson");
			paramMap.put("display", ResourceUtil.getString("edoc.edoctitle.createPerson.label"));
			columns.add(paramMap);
			//公文种类
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "docType");
			paramMap.put("display", ResourceUtil.getString("edoc.element.doctype"));
			columns.add(paramMap);
			//行文类型
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "sendType");
			paramMap.put("display", ResourceUtil.getString("edoc.element.sendtype"));
			columns.add(paramMap);
			//内部文号
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "serialNo");
			paramMap.put("display", ResourceUtil.getString("govdoc.serialNo.label"));
			columns.add(paramMap);
			//发文单位
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "sendUnit");
			paramMap.put("display", ResourceUtil.getString("edoc.element.sendunit"));
			columns.add(paramMap);
			//是否归档
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "hasArchive");
			paramMap.put("display", ResourceUtil.getString("govdoc.canArchive.label"));
			columns.add(paramMap);
			//归档路径
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "archiveName");
			paramMap.put("display", ResourceUtil.getString("govdoc.archiveName.label"));
			columns.add(paramMap);
			//抄送份数
			paramMap = new HashMap<String, String>();
			paramMap.put("name", "copies");
			paramMap.put("display", ResourceUtil.getString("govdoc.copies.label"));
			columns.add(paramMap);
		}
		return columns;
	}
	
	/**
	 * 公文查询时从前端Request中获取数据
	 * @param request
	 * @return
	 */
    public static Map<String, String> getConditionsByRequest(HttpServletRequest request) {
    	Map<String, String> params = new HashMap<String, String>();
    	params.put("listType", GovdocParamUtil.getString(request, "listType"));
    	params.put("govdocType", GovdocParamUtil.getString(request, "govdocType"));
    	params.put("statTitle", GovdocParamUtil.getString(request, "statTitle"));
    	params.put("condition", GovdocParamUtil.getString(request, "condition"));
    	params.put("deduplication", GovdocParamUtil.getString(request, "deduplication"));
    	params.put("subject", GovdocParamUtil.getString(request, "subject"));
    	params.put("serialNo", GovdocParamUtil.getString(request, "serialNo"));
    	params.put("createPerson", GovdocParamUtil.getString(request, "createPerson"));
    	params.put("sendUnit", GovdocParamUtil.getString(request, "sendUnit"));
    	params.put("sendDepartment", GovdocParamUtil.getString(request, "sendDepartment"));
    	params.put("sendTo", GovdocParamUtil.getString(request, "sendTo"));
    	params.put("printer", GovdocParamUtil.getString(request, "printer"));
    	params.put("issuer", GovdocParamUtil.getString(request, "issuer"));
    	params.put("review", GovdocParamUtil.getString(request, "review"));
    	params.put("auditor", GovdocParamUtil.getString(request, "auditor"));
    	params.put("signPerson", GovdocParamUtil.getString(request, "signPerson"));
    	params.put("undertaker", GovdocParamUtil.getString(request, "undertaker"));
    	params.put("undertakenoffice", GovdocParamUtil.getString(request, "undertakenoffice"));
    	params.put("printUnit", GovdocParamUtil.getString(request, "printUnit"));
    	params.put("copies", GovdocParamUtil.getString(request, "copies"));
    	params.put("sendType", GovdocParamUtil.getString(request, "sendType"));
    	params.put("docType", GovdocParamUtil.getString(request, "docType"));
    	params.put("keepPeriod", GovdocParamUtil.getString(request, "keepPeriod"));
    	params.put("urgentLevel", GovdocParamUtil.getString(request, "urgentLevel"));
    	params.put("unitLevel", GovdocParamUtil.getString(request, "unitLevel"));
    	params.put("secretLevel", GovdocParamUtil.getString(request, "secretLevel"));
    	params.put("customColumns", GovdocParamUtil.getString(request, "customColumns"));
    	params.put("customDisplayColumns", GovdocParamUtil.getString(request, "customDisplayColumns"));
    	
    	params.put("docMark", GovdocParamUtil.getString(request, "docMark"));	
    	params.put("startTime", GovdocParamUtil.getString(request, "startTime"));
    	params.put("createTime", GovdocParamUtil.getString(request, "createTime"));
    	params.put("signingDate", GovdocParamUtil.getString(request, "signingDate"));
    	params.put("packTime", GovdocParamUtil.getString(request, "packTime"));
    	params.put("recieveDate", GovdocParamUtil.getString(request, "recieveDate"));
    	params.put("registerDate", GovdocParamUtil.getString(request, "registerDate"));
    	
    	
    	//公文查询支持全量公文元素
    	params.put("copy_to", GovdocParamUtil.getString(request, "copy_to"));
    	params.put("report_to", GovdocParamUtil.getString(request, "report_to"));
    	params.put("createdate", GovdocParamUtil.getString(request, "createdate"));
    	params.put("filefz", GovdocParamUtil.getString(request, "filefz"));
    	params.put("phone", GovdocParamUtil.getString(request, "phone"));
    	params.put("receive_unit", GovdocParamUtil.getString(request, "receive_unit"));
    	params.put("sign_mark", GovdocParamUtil.getString(request, "sign_mark"));
    	
    	params.put("isNeedSearchSummaryExtend", GovdocParamUtil.getString(request, "isNeedSearchSummaryExtend"));
    	params.put("string1", GovdocParamUtil.getString(request, "string1"));
    	params.put("string2", GovdocParamUtil.getString(request, "string2"));
    	params.put("string3", GovdocParamUtil.getString(request, "string3"));
    	params.put("string4", GovdocParamUtil.getString(request, "string4"));
    	params.put("string5", GovdocParamUtil.getString(request, "string5"));
    	params.put("string6", GovdocParamUtil.getString(request, "string6"));
    	params.put("string7", GovdocParamUtil.getString(request, "string7"));
    	params.put("string8", GovdocParamUtil.getString(request, "string8"));
    	params.put("string9", GovdocParamUtil.getString(request, "string9"));
    	params.put("string10", GovdocParamUtil.getString(request, "string10"));
    	params.put("string11", GovdocParamUtil.getString(request, "string11"));
    	params.put("string12", GovdocParamUtil.getString(request, "string12"));
    	params.put("string13", GovdocParamUtil.getString(request, "string13"));
    	params.put("string14", GovdocParamUtil.getString(request, "string14"));
    	params.put("string15", GovdocParamUtil.getString(request, "string15"));
    	params.put("string16", GovdocParamUtil.getString(request, "string16"));
    	params.put("string17", GovdocParamUtil.getString(request, "string17"));
    	params.put("string18", GovdocParamUtil.getString(request, "string18"));
    	params.put("string19", GovdocParamUtil.getString(request, "string19"));
    	params.put("string20", GovdocParamUtil.getString(request, "string20"));
    	params.put("string21", GovdocParamUtil.getString(request, "string21"));
    	params.put("string22", GovdocParamUtil.getString(request, "string22"));
    	params.put("string23", GovdocParamUtil.getString(request, "string23"));
    	params.put("string24", GovdocParamUtil.getString(request, "string24"));
    	params.put("string25", GovdocParamUtil.getString(request, "string25"));
    	params.put("string26", GovdocParamUtil.getString(request, "string26"));
    	params.put("string27", GovdocParamUtil.getString(request, "string27"));
    	params.put("string28", GovdocParamUtil.getString(request, "string28"));
    	params.put("string29", GovdocParamUtil.getString(request, "string29"));
    	params.put("string30", GovdocParamUtil.getString(request, "string30"));
    	params.put("integer1", GovdocParamUtil.getString(request, "integer1"));
    	params.put("integer2", GovdocParamUtil.getString(request, "integer2"));
    	params.put("integer3", GovdocParamUtil.getString(request, "integer3"));
    	params.put("integer4", GovdocParamUtil.getString(request, "integer4"));
    	params.put("integer5", GovdocParamUtil.getString(request, "integer5"));
    	params.put("integer6", GovdocParamUtil.getString(request, "integer6"));
    	params.put("integer7", GovdocParamUtil.getString(request, "integer7"));
    	params.put("integer8", GovdocParamUtil.getString(request, "integer8"));
    	params.put("integer9", GovdocParamUtil.getString(request, "integer9"));
    	params.put("integer10", GovdocParamUtil.getString(request, "integer10"));
    	params.put("integer11", GovdocParamUtil.getString(request, "integer11"));
    	params.put("integer12", GovdocParamUtil.getString(request, "integer12"));
    	params.put("integer13", GovdocParamUtil.getString(request, "integer13"));
    	params.put("integer14", GovdocParamUtil.getString(request, "integer14"));
    	params.put("integer15", GovdocParamUtil.getString(request, "integer15"));
    	params.put("integer16", GovdocParamUtil.getString(request, "integer16"));
    	params.put("integer17", GovdocParamUtil.getString(request, "integer17"));
    	params.put("integer18", GovdocParamUtil.getString(request, "integer18"));
    	params.put("integer19", GovdocParamUtil.getString(request, "integer19"));
    	params.put("integer20", GovdocParamUtil.getString(request, "integer20"));
    	params.put("decimal1", GovdocParamUtil.getString(request, "decimal1"));
    	params.put("decimal2", GovdocParamUtil.getString(request, "decimal2"));
    	params.put("decimal3", GovdocParamUtil.getString(request, "decimal3"));
    	params.put("decimal4", GovdocParamUtil.getString(request, "decimal4"));
    	params.put("decimal5", GovdocParamUtil.getString(request, "decimal5"));
    	params.put("decimal6", GovdocParamUtil.getString(request, "decimal6"));
    	params.put("decimal7", GovdocParamUtil.getString(request, "decimal7"));
    	params.put("decimal8", GovdocParamUtil.getString(request, "decimal8"));
    	params.put("decimal9", GovdocParamUtil.getString(request, "decimal9"));
    	params.put("decimal10", GovdocParamUtil.getString(request, "decimal10"));
    	params.put("decimal11", GovdocParamUtil.getString(request, "decimal11"));
    	params.put("decimal12", GovdocParamUtil.getString(request, "decimal12"));
    	params.put("decimal13", GovdocParamUtil.getString(request, "decimal13"));
    	params.put("decimal14", GovdocParamUtil.getString(request, "decimal14"));
    	params.put("decimal15", GovdocParamUtil.getString(request, "decimal15"));
    	params.put("decimal16", GovdocParamUtil.getString(request, "decimal16"));
    	params.put("decimal17", GovdocParamUtil.getString(request, "decimal17"));
    	params.put("decimal18", GovdocParamUtil.getString(request, "decimal18"));
    	params.put("decimal19", GovdocParamUtil.getString(request, "decimal19"));
    	params.put("decimal20", GovdocParamUtil.getString(request, "decimal20"));
    	params.put("date1", GovdocParamUtil.getString(request, "date1"));
    	params.put("date2", GovdocParamUtil.getString(request, "date2"));
    	params.put("date3", GovdocParamUtil.getString(request, "date3"));
    	params.put("date4", GovdocParamUtil.getString(request, "date4"));
    	params.put("date5", GovdocParamUtil.getString(request, "date5"));
    	params.put("date6", GovdocParamUtil.getString(request, "date6"));
    	params.put("date7", GovdocParamUtil.getString(request, "date7"));
    	params.put("date8", GovdocParamUtil.getString(request, "date8"));
    	params.put("date9", GovdocParamUtil.getString(request, "date9"));
    	params.put("date10", GovdocParamUtil.getString(request, "date10"));
    	params.put("date11", GovdocParamUtil.getString(request, "date11"));
    	params.put("date12", GovdocParamUtil.getString(request, "date12"));
    	params.put("date13", GovdocParamUtil.getString(request, "date13"));
    	params.put("date14", GovdocParamUtil.getString(request, "date14"));
    	params.put("date15", GovdocParamUtil.getString(request, "date15"));
    	params.put("date16", GovdocParamUtil.getString(request, "date16"));
    	params.put("date17", GovdocParamUtil.getString(request, "date17"));
    	params.put("date18", GovdocParamUtil.getString(request, "date18"));
    	params.put("date19", GovdocParamUtil.getString(request, "date19"));
    	params.put("date20", GovdocParamUtil.getString(request, "date20"));
    	params.put("list1", GovdocParamUtil.getString(request, "list1"));
    	params.put("list2", GovdocParamUtil.getString(request, "list2"));
    	params.put("list3", GovdocParamUtil.getString(request, "list3"));
    	params.put("list4", GovdocParamUtil.getString(request, "list4"));
    	params.put("list5", GovdocParamUtil.getString(request, "list5"));
    	params.put("list6", GovdocParamUtil.getString(request, "list6"));
    	params.put("list7", GovdocParamUtil.getString(request, "list7"));
    	params.put("list8", GovdocParamUtil.getString(request, "list8"));
    	params.put("list9", GovdocParamUtil.getString(request, "list9"));
    	params.put("list10", GovdocParamUtil.getString(request, "list10"));
    	params.put("list11", GovdocParamUtil.getString(request, "list11"));
    	params.put("list12", GovdocParamUtil.getString(request, "list12"));
    	params.put("list13", GovdocParamUtil.getString(request, "list13"));
    	params.put("list14", GovdocParamUtil.getString(request, "list14"));
    	params.put("list15", GovdocParamUtil.getString(request, "list15"));
    	params.put("list16", GovdocParamUtil.getString(request, "list16"));
    	params.put("list17", GovdocParamUtil.getString(request, "list17"));
    	params.put("list18", GovdocParamUtil.getString(request, "list18"));
    	params.put("list19", GovdocParamUtil.getString(request, "list19"));
    	params.put("list20", GovdocParamUtil.getString(request, "list20"));
    	params.put("text1", GovdocParamUtil.getString(request, "text1"));
    	params.put("text2", GovdocParamUtil.getString(request, "text2"));
    	params.put("text3", GovdocParamUtil.getString(request, "text3"));
    	params.put("text4", GovdocParamUtil.getString(request, "text4"));
    	params.put("text5", GovdocParamUtil.getString(request, "text5"));
    	params.put("text6", GovdocParamUtil.getString(request, "text6"));
    	params.put("text7", GovdocParamUtil.getString(request, "text7"));
    	params.put("text8", GovdocParamUtil.getString(request, "text8"));
    	params.put("text9", GovdocParamUtil.getString(request, "text9"));
    	params.put("text10", GovdocParamUtil.getString(request, "text10"));
    	params.put("text11", GovdocParamUtil.getString(request, "text11"));
    	params.put("text12", GovdocParamUtil.getString(request, "text12"));
    	params.put("text13", GovdocParamUtil.getString(request, "text13"));
    	params.put("text14", GovdocParamUtil.getString(request, "text14"));
    	params.put("text15", GovdocParamUtil.getString(request, "text15"));
    	return params;
    }
    
    /**
     * 计算节点处理剩余时间
     * @param date 流程到达时间
     * @param expecetProcessTime 节点处理期限（具体时间）
     * @return 剩余时间（不足一天单位为：小时，反之单位为：天）
     */
    @SuppressWarnings("deprecation")
    public static int[] calculateSurplusTime(java.util.Date date,java.util.Date expecetProcessTime) throws BusinessException {
        int[] surplusTime = null;
        User user = AppContext.getCurrentUser();
        try {
            if(expecetProcessTime !=null) {
                long days = 0;
                long hours = 0;
                long minutes = 0;
                // 获取系统当前时间
                java.util.Date nowTime = new java.util.Date();
                // 得到节点处理的最后时间
                //java.util.Date overTime = workTimeManager.getCompleteDate4Nature(date, deadlineDate, user.getAccountId());
                // 未超期
                if(nowTime.before(expecetProcessTime)) {
                    // 得到剩余处理时间（分钟）
                	long surplusMinu = workTimeManager.getDealWithTimeValue(nowTime, expecetProcessTime, user.getAccountId()) / (1000 * 60);
                    // 得到当前单位当月的日工作时间（分钟）
                    long dayOfMinu = workTimeManager.getEachDayWorkTime(nowTime.getYear(), user.getAccountId());
                    if(surplusMinu >= dayOfMinu) {
                        // 天数
                        days = surplusMinu / dayOfMinu;
                        long shenyufen = surplusMinu - days * dayOfMinu;
                        if(shenyufen < 60) {
                            minutes = shenyufen;
                        } else {
                            hours = shenyufen / 60;
                            minutes = shenyufen - hours * 60;
                        }
                    } else if(60 <= surplusMinu && surplusMinu < dayOfMinu) {
                        hours = surplusMinu / 60;
                        minutes = surplusMinu - hours * 60;
                    } else {
                        minutes = surplusMinu;
                    }
                }
                surplusTime = new int[3];
                surplusTime[0]=(int)days;
                surplusTime[1]=(int)hours;
                surplusTime[2]=(int)minutes;
            }
        } catch(WorkTimeSetExecption e) {
            //log.error("计算节点处理剩余时间抛出异常", e);
        }
        return surplusTime;
    }
	
    /**
	 * 列表查询数据-回填代理人
	 * @param vo
	 * @param showProxyInfo4A
	 * @param _userId
	 * @param listType
	 * @throws BusinessException
	 */
	public static void fillListAgent(GovdocListVO vo,boolean showProxyInfo4A,Long _userId,String listType) throws BusinessException{
    	Long userId = _userId;
    	if(_userId == null){
    		userId = AppContext.getCurrentUser().getId();
        }
        /*
         * A设B为代理
         *
         * B登录，获取我的被代理人A的事项，并把这些事项标记为蓝色（只要“事项的MemberId!=我”即可）
         * A登录，仅按照"member=我的id"即可，但要把我代理出去的事项标记为蓝色（事项是否落入到代理条件内）
         *
         * 查询我的代理列表(我为被代理人，我找别人干活)   key: 模板ID,自由协同为AllSelf,全部模板为AllTemplate
         */
        Map<Object, AgentModel> agentToModelColl = new HashMap<Object, AgentModel>();
        if(showProxyInfo4A){ //A登录
        	//查询我的代理人
            List<AgentModel> agentToModelList = MemberAgentBean.getInstance().getAgentModelToList(userId);
            if(!Strings.isEmpty(agentToModelList)){
                for (AgentModel agentTo : agentToModelList) {
                    if(agentTo.getStartDate().before(new Date()) && agentTo.getEndDate().after(new Date())){
                        if(agentTo.isHasCol()){
                        	agentToModelColl.put("AllSelf", agentTo);  //表示自由协同
                        }
                        if(agentTo.isHasEdocFree() || agentTo.isHasEdocTemplate()){
                        	agentToModelColl.put("EdocAllSelf", agentTo);  //表示公文
                        }
                        if(agentTo.isHasTemplate()){
                            List<AgentDetailModel> models = agentTo.getAgentDetail(ApplicationCategoryEnum.collaboration.key());
                            if(!Strings.isEmpty(models)){
                                for (AgentDetailModel agentDetailModel : models) {
                                    agentToModelColl.put(agentDetailModel.getEntityId(), agentTo);  //指定模板
                                }
                            }
                            else{
                                agentToModelColl.put("AllTemplate", agentTo);  //表示全部模板
                            }
                        }
                    }
                }
            }
            Long templateId = vo.getTempleteId();
            if(!vo.getAffairMemberId().equals(userId)){
            	vo.setProxyName(Functions.showMemberNameOnly(vo.getAffairMemberId()));
            	vo.setProxy(true);
            }
            if(vo.getAffairTransactorId()!=null && vo.getAffairTransactorId().longValue() != userId.longValue()){
            	vo.setProxyName(Functions.showMemberNameOnly(vo.getAffairTransactorId()));
            	vo.setProxy(true);
            }
            else if(agentToModelColl != null && !agentToModelColl.isEmpty()){
                Date early = null;
                if(templateId == null && agentToModelColl.get("AllSelf") != null){ //自由协同
                    early = agentToModelColl.get("AllSelf").getStartDate();
                }
                else if(templateId != null && agentToModelColl.get("AllTemplate") != null){
                    early = agentToModelColl.get("AllTemplate").getStartDate();
                }
                else if(templateId != null && agentToModelColl.get(templateId) != null){
                    early = agentToModelColl.get(templateId).getStartDate();
                }
                else if(agentToModelColl.get("EdocAllSelf") != null){
                	early = agentToModelColl.get("EdocAllSelf").getStartDate();
                }
                if(vo.getAffairReceiveTime() != null){//BDGW-2193代理人设置之后，代理人的已办公文报错
                	if(early != null && early.before(vo.getAffairReceiveTime())){
                        vo.setProxy(true);
                    }
                }
            }
            String showAgent = "";
			if(listType.contains("Pending") || listType.contains("Done") || listType.contains("Finished")) {
				showAgent = GovdocListHelper.showToPendingAgent(vo);
			}
			if(!vo.getSubject().contains(ResourceUtil.getString("govdoc.generation.record"))){
	            vo.setSubject(vo.getSubject() + showAgent);
	            vo.setAffairSubject(vo.getAffairSubject() + showAgent);
			}
        }
	}
    
    /**
     * 拼接代理标识
     * @param vo
     * @return
     * @throws BusinessException 
     */
    public static String showToPendingAgent(GovdocListVO vo) throws BusinessException{
        if(Strings.isEmpty(vo.getProxyName())){
            return "";
        }
        String colProxyLabel = "";
        if(Boolean.TRUE.equals(vo.getProxy())){
        	//我的代理列表(我代表别人)，我是代理人
            List<AgentModel> _agentModelList = MemberAgentBean.getInstance().getAgentModelList(AppContext.getCurrentUser().getId());
            //代理我的代理人列表，我是被代理人
            List<AgentModel> _agentModelToList = MemberAgentBean.getInstance().getAgentModelToList(AppContext.getCurrentUser().getId());
            boolean agentToFlag = false;
            if(_agentModelList != null && !_agentModelList.isEmpty()){
                agentToFlag = false;
            }else if(_agentModelToList != null && !_agentModelToList.isEmpty()){
                agentToFlag = true;
            }
            if(agentToFlag){
            	
                CtpAffair affair = affairManager.get(vo.getAffairId());
            	if (affair == null) {
            		affair = affairManager.getByHis(vo.getAffairId());
				}
            	Map<String, Object> map = AffairUtil.getExtProperty(affair);
            	if(map.get(AffairExtPropEnums.dailu_pishi_mark.toString()) != null 
            			&& !"".equals(map.get(AffairExtPropEnums.dailu_pishi_mark.toString()))){
            		//(xx)代录
            		colProxyLabel = "(" + vo.getProxyName() +  ResourceUtil.getString("govdoc.generation.record");
            	}else{
            		//(XX)代理
                    colProxyLabel = "(" + vo.getProxyName() + PROXYSTR + ")";
            	}
            }else{
                if(vo.getAgentDeal()){      //被代理人自己处理
                	//(XX)处理
                    colProxyLabel = "(" + ResourceUtil.getString("collaboration.proxy.deal", vo.getProxyName()) + ")";
                }else{
                	//add by rz 2017-08-19 [添加代录人权限判断]
                	CtpAffair affair = affairManager.get(vo.getAffairId());
                	if (affair == null) {
                		affair = affairManager.getByHis(vo.getAffairId());
					}
                	Map<String, Object> map = AffairUtil.getExtProperty(affair);
                	if(map.get(AffairExtPropEnums.dailu_pishi_mark.toString()) != null 
                			&& !"".equals(map.get(AffairExtPropEnums.dailu_pishi_mark.toString()))){
                		//(xx)代录
                		colProxyLabel = "(" + vo.getProxyName() +  ResourceUtil.getString("govdoc.generation.record");
                	}else{
                		//代理(XX)
                		colProxyLabel = "(" + vo.getProxyName() + ResourceUtil.getString("collaboration.proxy") + ")";
                	}
                }
            }
        }
        return colProxyLabel;
    }
    
    public static String showToDoneAgent(GovdocListVO vo){

        V3xOrgMember member = null;
        String colProxyLabel = "";
        boolean isAgent = true;
        User user = AppContext.getCurrentUser();
        long userId = user.getId();
        if (vo.getAffairTransactorId() != null || vo.getProxyMemberId() != null) {// 当前登录人就是代理人的时候不需要显示代理信息
			Long memberId = vo.getAffairTransactorId();
			if (memberId != null && memberId.longValue() == userId) {
				memberId = vo.getAffairMemberId();
			} else if (vo.getProxyMemberId() != null && vo.getProxyMemberId().longValue() == userId) {
				memberId = vo.getProxyMemberId();
			} else {
				isAgent = false;
			}
            try {
                member = orgManager.getMemberById(memberId);
                if(!member.getIsAdmin()){
                  if(member != null){
                    if(isAgent){
                      //代理(XX)
                      colProxyLabel = "(" + ResourceUtil.getString("collaboration.proxy") + member.getName() + ")";
                    }else {
                    	//add by rz 2017-08-19 [添加代录人权限判断]
                    	CtpAffair affair = affairManager.get(vo.getAffairId());
                    	Map<String, Object> map = AffairUtil.getExtProperty(affair);
                    	if(map.get(AffairExtPropEnums.dailu_pishi_mark.toString()) != null 
                    			&& !"".equals(map.get(AffairExtPropEnums.dailu_pishi_mark.toString()))){
                    		//(xx)代录
                    		colProxyLabel = "(" + member.getName() +  ResourceUtil.getString("govdoc.generation.record");
                    	}else{
                    		//(XX)代理
                    		colProxyLabel = "(" + member.getName() + ResourceUtil.getString("collaboration.proxy") + ")";
                    	}
                    }
                  }
                }

            } catch (Exception e) {
            }
          }else if(vo.getAffairMemberId() != userId){//代理人已办查看被代理人处理的
        	try{
    			member = orgManager.getMemberById(vo.getAffairMemberId());
    			//(XX)处理
                colProxyLabel = "(" + ResourceUtil.getString("collaboration.proxy.deal", member.getName()) + ")";
    		}catch(Exception e){
    		}
        }else{
        	vo.setProxy(false);
        }
        return colProxyLabel;
    
    }
    /**
     * @Title: isAllShouWenTemplate
     * @Description: 检查模板是否包含收文模板-用于业务生成器公文模板列表展示
     * @throws BusinessException 
     */
    public static boolean isContainRecTemplate(String templateIds) throws BusinessException {
    	if(Strings.isNotBlank(templateIds)) {
			List<Long> templateIdList = new ArrayList<Long>();
			String[] templateIdArr = templateIds.split(",");
			if(templateIdArr != null) {
				for(int i=0; i<templateIdArr.length; i++) {
					templateIdList.add(Long.parseLong(templateIdArr[i]));
				}
			}
			List<CtpTemplate> templateList=templateManager.getTemplatesByIds(templateIdList);
			for(CtpTemplate template:templateList){
				if(template.getModuleType()!=null && template.getModuleType().intValue()==ApplicationCategoryEnum.govdocRec.getKey()){
					//只要其中有一个收文模板，则返回true
					return true;
				}
			}
		}
    	return false;
    }
    
    /**
     * 设置列表的归档显示值
     * @param listVo
     * @param docPathMap 
     * @throws BusinessException
     */
    public static void fillListArchiveData(GovdocListVO listVo, Map<String, String> docPathMap) throws BusinessException {
    	List<DocResourceBO> docs = null;
    	if(AppContext.hasPlugin("doc")){
    		docs = docApi.findDocResourcesBySourceId(listVo.getAffairId());
        	//这里排序的目的是，因为底层没有排序，所以这里拿出来的顺序可能不一样
        	Collections.sort(docs,new Comparator<DocResourceBO>() {
    			@Override
    			public int compare(DocResourceBO o1, DocResourceBO o2) {
    				return (int) (o1.getFrType() - o2.getFrType());
    			}
    		});
    	}
   		if(Strings.isNotEmpty(docs)) {
   			String key = "";
   			for(DocResourceBO bo : docs){
   				key += bo.getId();
   			}
   			if(docPathMap.containsKey(key)){
   				listVo.setArchiveName(docPathMap.get(key));
   				return;
   			}
   			String frName = "";
   			for(DocResourceBO bo : docs){
   				bo = docApi.getDocResource(bo.getParentFrId());      				
   				String tempFrName=bo.getFrName();
   				//获取完整文字路径
   				String fullPath= docApi.getPhysicalPath(bo.getLogicalPath(), "\\", false, 0);
   				if (GovdocHelper.needI18n(bo.getFrType())){
   					tempFrName = ResourceUtil.getString(tempFrName);
   				}
   				
   				if(bo.getLogicalPath()!=null && bo.getLogicalPath().split("\\.").length>1){
   					tempFrName=fullPath;
   				}
   				frName = frName + "|"+tempFrName;
   			}
   			frName = frName.length() > 1 ? frName.substring(1) : "";
   			listVo.setArchiveName(frName);
   			docPathMap.put(key, frName);
   		} else {
   			if(listVo.getHasArchive()){
   				listVo.setHasArchive(true);
   				if (listVo.getArchiveId() != null) {
   				    //公文查询-直接根据ArchiveId查询归档路径
   	   				DocResourceBO bo = docApi.getDocResource(listVo.getArchiveId());
   	   				String tempFrName=bo.getFrName();
   	   				//获取完整文字路径
   	   				String fullPath= docApi.getPhysicalPath(bo.getLogicalPath(), "\\", false, 0);
   	   				if (GovdocHelper.needI18n(bo.getFrType())){
   	   					tempFrName = ResourceUtil.getString(tempFrName);
   	   				}
   	   				if(bo.getLogicalPath()!=null && bo.getLogicalPath().split("\\.").length>1){
   	   					tempFrName=fullPath;
   	   				}
   	   	   			listVo.setArchiveName(tempFrName);
				}
   			}else{
   				listVo.setHasArchive(false);
   			}
   		}
    }
    
    /**
     * 设置列表枚举显示值
     * @param listVo
     * @param docType
     * @param sendType
     * @param keepPeriod
     * @param secretLevel
     * @param urgentLevel
     * @param unitLevel
     */
    public static void fillListEnumLabel(GovdocListVO listVo, 
    		CtpEnumBean docType, CtpEnumBean sendType, CtpEnumBean keepPeriod, 
    		CtpEnumBean secretLevel,  CtpEnumBean urgentLevel, CtpEnumBean unitLevel) {
    	//公文种类
        if(Strings.isNotBlank(listVo.getDocType())) {
        	for(com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem enmu : docType.getItems()) {
        		if(listVo.getDocType().equals(enmu.getEnumvalue())) {
        			listVo.setDocType(ResourceUtil.getString(enmu.getShowvalue()));
        			break;
        		}
        	}
		}
        //行文类型
        if(Strings.isNotBlank(listVo.getSendType())) {
        	for(com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem enmu : sendType.getItems()) {
        		if(listVo.getSendType().equals(enmu.getEnumvalue())) {
        			listVo.setSendType(ResourceUtil.getString(enmu.getShowvalue()));
        			break;
        		}
        	}
		}
        //密级
        if(Strings.isNotBlank(listVo.getSecretLevel())) {
        	for(com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem enmu : secretLevel.getItems()) {
        		if(listVo.getSecretLevel().equals(enmu.getEnumvalue())) {
        			listVo.setSecretLevel(ResourceUtil.getString(enmu.getShowvalue()));
        			break;
        		}
        	}
		}
		//紧急程度
       	if(Strings.isNotBlank(listVo.getUrgentLevel())) {
        	for(com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem enmu : urgentLevel.getItems()) {
    			if(listVo.getUrgentLevel().equals(enmu.getEnumvalue())) {
					 listVo.setUrgentLevel(ResourceUtil.getString(enmu.getShowvalue()));
					 break;
    			}
    		}
        }
       	//公文类别
       	if(Strings.isNotBlank(listVo.getUnitLevel())) {
        	for(com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem enmu : unitLevel.getItems()) {
    			if(listVo.getUnitLevel().equals(enmu.getEnumvalue())) {
					 listVo.setUnitLevel(ResourceUtil.getString(enmu.getShowvalue()));
					 break;
    			}
    		}
        }
       	//保密期限
       	if(listVo.getKeepPeriod() != null) {
        	for(com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem enmu : keepPeriod.getItems()) {
        		if(Strings.isNotEmpty(enmu.getEnumvalue())) {
	    			if(listVo.getKeepPeriod().intValue() == Integer.parseInt(enmu.getEnumvalue())) {
	    				listVo.setKeepPeriodTxt(ResourceUtil.getString(enmu.getShowvalue()));
						 break;
	    			}
        		}
    		}
        }
    }
    
    /**
	 * 获取当前公文的全部批示编号
     * @param leaderPishiMap 
     * @param summaryIds 
	 * @param summaryId
	 * @return
	 */
	public static String getAllLeaderPishi(Map<Long, String> leaderPishiMap, List<Long> summaryIds, Long summaryId) throws BusinessException {
		String allLeaderPishi = "";
		if(leaderPishiMap.isEmpty()){
			//防止summaryids太多，每次处理500条
			for (int i = 0; i <= summaryIds.size() / 500; i++) {
				int fromIndex = i * 500;
				int toIndex = Math.min(summaryIds.size(), (i + 1) * 500);
				List<EdocLeaderPishiNo> list = govdocPishiManager.getAllLeaderPishiBySummaryIds(summaryIds.subList(fromIndex, toIndex));
				Long key = 0L;
				for (EdocLeaderPishiNo leaderPishiNo : list) {
					if(key.longValue() != leaderPishiNo.getSummaryId().longValue()){
						leaderPishiMap.put(key, allLeaderPishi);
						key = leaderPishiNo.getSummaryId();
						allLeaderPishi="";
					}
					if (leaderPishiNo.getPishiName() != null && leaderPishiNo.getPishiNo() != null) {
						allLeaderPishi += leaderPishiNo.getPishiName() + " ";
						allLeaderPishi += leaderPishiNo.getPishiYear() + ResourceUtil.getString("govdoc.year");
						allLeaderPishi += ResourceUtil.getString("govdoc.NO.") + leaderPishiNo.getPishiNo() + ResourceUtil.getString("govdoc.number");
						allLeaderPishi += "	";
					}
					
				}
				leaderPishiMap.put(key, allLeaderPishi);
			}
		}
		return leaderPishiMap.get(summaryId);
	}
	
	/**
	 * 公文查询、公文统计获取查询条件和自定义查询显示元素
	 * @param 
	 * @param 
	 * @return
	 */
	public static List<EdocElement> getEdocQueryAndStatDisplayElement(List<EdocElement> allElements , String dialogType, String formType) {
		List<EdocElement> resultList = new ArrayList<EdocElement>();
		for (EdocElement item : allElements) {
				//过滤公文元素-附件、附件说明(过滤处理意见元素)
			if ("niwen".equals(item.getFieldName())
					|| "shenpi".equals(item.getFieldName())
					|| "shenhe".equals(item.getFieldName())
					|| "fuhe".equals(item.getFieldName())
					|| "fengfa".equals(item.getFieldName())
					|| "huiqian".equals(item.getFieldName())
					|| "qianfa".equals(item.getFieldName())
					|| "zhihui".equals(item.getFieldName())
					|| "yuedu".equals(item.getFieldName())
					|| "banli".equals(item.getFieldName())
					|| "dengji".equals(item.getFieldName())
					|| "niban".equals(item.getFieldName())
					|| "pishi".equals(item.getFieldName())
					|| "chengban".equals(item.getFieldName())
					|| "otherOpinion".equals(item.getFieldName())
					|| "wenshuguanli".equals(item.getFieldName())
					|| "report".equals(item.getFieldName())
					|| "feedback".equals(item.getFieldName())
					|| "attachments".equals(item.getFieldName())
					|| "nibanyijian".equals(item.getFieldName())
					|| "receive_unit".equals(item.getFieldName())
					|| "filefz".equals(item.getFieldName())
					|| item.getFieldName().startsWith("string")
					|| item.getFieldName().startsWith("integer")
					|| item.getFieldName().startsWith("decimal")
					|| item.getFieldName().startsWith("date")
					|| item.getFieldName().startsWith("text")
					|| "filesm".equals(item.getFieldName())) {
				
			}else{
				//报批意见、抄送单位、抄报单位在查询条件中屏蔽，在自定义列表中显示
				if("conditonDialog".equals(dialogType)&&("baopiyijian".equals(item.getFieldName()) || "copy_to".equals(item.getFieldName()) || "report_to".equals(item.getFieldName()))){
					continue;
				//发文登记簿、签报登记簿不显示签收编号
				}else if("sign_mark".equals(item.getFieldName()) && ("listSendRegister".equals(formType)||"listSignRegister".equals(formType))){
					continue;
				}
				else if("createdate".equals(item.getFieldName()) && "edocQuery".equals(formType)){
					//公文查询：拟稿日期------->发起日期 
					EdocElement dateItem = new EdocElement();
					dateItem.setName(ResourceUtil.getString("govdoc.query.startDate"));
					dateItem.setFieldName("createdate");
					resultList.add(dateItem);
				}
				else if("send_department".equals(item.getFieldName()) && ("listSendRegister".equals(formType) || "listSignRegister".equals(formType))){
					//发文、签报登记簿：发文部门-------->拟稿部门
					EdocElement dateItem = new EdocElement();
					dateItem.setName(ResourceUtil.getString("govdoc.element.senddepartment3"));
					dateItem.setFieldName("send_department");
					resultList.add(dateItem);
				}
				else if("send_unit".equals(item.getFieldName()) &&  "listRecRegister".equals(formType)){
					//收文登记簿：发文单位 ----> 来文单位
					EdocElement dateItem = new EdocElement();
					dateItem.setName(ResourceUtil.getString("govdoc.communication.unit"));
					dateItem.setFieldName("send_unit");
					resultList.add(dateItem);
				}
				else if("createdate".equals(item.getFieldName()) &&  ("listSendRegister".equals(formType) || "listSignRegister".equals(formType) || "listRecRegister".equals(formType))){
					//发文登记簿、收文登记簿、签报登记簿 拟稿日期-->拟文日期
					EdocElement dateItem = new EdocElement();
					dateItem.setName(ResourceUtil.getString("edoc.edoctitle.createDate.label"));
					dateItem.setFieldName("createdate");
					resultList.add(dateItem);
				}
				else{
					EdocElement resultItem = new EdocElement();
					resultItem.setElementId(item.getElementId());
					resultItem.setName(ResourceUtil.getString(item.getName()));
					resultItem.setFieldName(item.getFieldName());
					resultItem.setType(item.getType());
					resultList.add(resultItem);
				}
			}
		}
		    //增加非公文元素 是否归档、归档路径
		EdocElement hasArchiveItem = new EdocElement();
		hasArchiveItem.setName(ResourceUtil.getString("govdoc.canArchive.label"));
		hasArchiveItem.setFieldName("hasArchive");
		resultList.add(hasArchiveItem);
		//公文查询自定义才查才显示归档路径，不作为条件查询，公文统计不查归档路径
		if("edocQuery".equals(formType) && "querySettingDialog".equals(dialogType)){
			EdocElement archiveNameItem = new EdocElement();
			archiveNameItem.setName(ResourceUtil.getString("govdoc.archiveName.label"));
			archiveNameItem.setFieldName("archiveName");
			resultList.add(archiveNameItem);
		}
		
		if("listSendRegister".equals(formType)||"listRecRegister".equals(formType)||"listSignRegister".equals(formType)){
			//登记簿增加 领导批示编号字段
			EdocElement pishiItem = new EdocElement();
			pishiItem.setName(ResourceUtil.getString("govdoc.element.leaderCommondSort.label"));//领导批示编号
			pishiItem.setFieldName("pishiName");
			resultList.add(pishiItem);
			
			EdocElement currentNodesInfoItem = new EdocElement();
			currentNodesInfoItem.setName(ResourceUtil.getString("govdoc.currentNodesInfo.label"));//当前待办人
			currentNodesInfoItem.setFieldName("currentNodesInfo");
			resultList.add(currentNodesInfoItem);
			
			EdocElement summaryTransferStatusNameItem = new EdocElement();
			summaryTransferStatusNameItem.setName(ResourceUtil.getString("govdoc.stat.flowstat.label"));//流转状态
			summaryTransferStatusNameItem.setFieldName("summaryTransferStatusName");
			resultList.add(summaryTransferStatusNameItem);
		}
		
		if("listRecRegister".equals(formType)){ //收文登记簿
			EdocElement dateItem = new EdocElement();
			dateItem.setName(ResourceUtil.getString("edoc.element.communication.date"));//来文日期
			dateItem.setFieldName("receipt_date");
			resultList.add(dateItem);
			
			//增加分办人和分办日期
			EdocElement distributePerson = new EdocElement();
			distributePerson.setName(ResourceUtil.getString("govdoc.element.receive.distributer"));//分办人
			distributePerson.setFieldName("distributer");
			//resultList.add(distributePerson);
			
			EdocElement distributeDate = new EdocElement();
			distributeDate.setName(ResourceUtil.getString("edoc.edoctitle.disDate.label"));//分办日期
			distributeDate.setFieldName("createTime");
			resultList.add(distributeDate);
			
		}
		return resultList;
	}
	
	/**
	 * 公文查询、公文统计获取list元素绑定的枚举
	 * @param 
	 * @param 
	 * @return
	 */
	public static Map<String, CtpEnumBean> getEdocQueryAndStatListBindEnum(List<EdocElement> resultList) {
		Map<String, CtpEnumBean> listEnum = new HashMap<String, CtpEnumBean>();
		for(EdocElement itElement : resultList){
			if (itElement.getFieldName().startsWith("list")) {
			   EdocElement edocElement = govdocElementManager.getByFieldName(itElement.getFieldName());
			   if (edocElement.getMetadataId()!=null) {
				   CtpEnumBean listBindEnum = enumManagerNew.getEnum(edocElement.getMetadataId());
				    ArrayList<CtpEnumItem> allItems = (ArrayList<CtpEnumItem>) listBindEnum.getAllItems();
				    for (CtpEnumItem item : allItems) {
				    	//有些枚举需要国际化处理
				    	if(item.getShowvalue().contains(".")){
				    		item.setShowvalue(ResourceUtil.getString(item.getShowvalue()));
				    	}
					}
				    listBindEnum.setAllItems(allItems);
				    listEnum.put(edocElement.getFieldName(), listBindEnum);
			   }
		  }
		}
		return listEnum;
	}
	
	/**
	 * 公文查询、公文统计获取list枚举值
	 * @param 
	 * @param 
	 * @return
	 */
	public static Map<String, CtpEnumBean> getRegisterHistoryListEnum(List<ConfigItem> configItems , String currentkeyName) {
		Map<String, CtpEnumBean> listHistoryEnum = new HashMap<String, CtpEnumBean>();
		for(int i=0;i<configItems.size();i++){
    		ConfigItem configItem = configItems.get(i);
    		if (currentkeyName.equals(configItem.getConfigCategory())) {
    			if(configItem.getConfigValue()!=null && configItem.getConfigValue().length()>0){
    				  String[] array = configItem.getConfigValue().split(",");
    				  for (int j = 0; j < array.length; j++) {
						  String string = array[j];
						  if (Strings.isNotBlank(string)&& string.startsWith("list")) {
							  EdocElement edocElement = govdocElementManager.getByFieldName(string);
							  if (edocElement.getMetadataId() != null) {
								    CtpEnumBean listBindEnum = enumManagerNew.getEnum(edocElement.getMetadataId());
								    ArrayList<CtpEnumItem> allItems = (ArrayList<CtpEnumItem>) listBindEnum.getAllItems();
								    for (CtpEnumItem item : allItems) {
								    	//有些枚举需要国际化处理
								    	if(item.getShowvalue().contains(".")){
								    		item.setShowvalue(ResourceUtil.getString(item.getShowvalue()));
								    	}
									}
								    listBindEnum.setAllItems(allItems);
									listHistoryEnum.put(edocElement.getFieldName(), listBindEnum);
							   }
							  
						  }
					  }
    			}
				
			}	
    	}
		return listHistoryEnum;
	}
}
