package com.seeyon.apps.govdoc.helper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.govdoc.vo.GovdocWorkflowVO;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.organization.OrgConstants;
import com.seeyon.ctp.organization.bo.MemberPost;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgDepartment;
import com.seeyon.ctp.organization.bo.V3xOrgEntity;
import com.seeyon.ctp.organization.bo.V3xOrgLevel;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgPost;
import com.seeyon.ctp.organization.bo.V3xOrgRole;
import com.seeyon.ctp.organization.bo.V3xOrgTeam;
import com.seeyon.ctp.organization.dao.OrgHelper;
import com.seeyon.ctp.organization.po.OrgRole;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.workflow.constants.WorkFlowConstants;
import com.seeyon.ctp.workflow.dynamicform.util.RoleTransferUtil;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.v3x.edoc.domain.EdocMarkAcl;
import com.seeyon.v3x.edoc.domain.EdocObjTeam;
import com.seeyon.v3x.exchange.domain.ExchangeAccount;

/**
 * 公文角色相关辅助类
 * 
 * @author 唐桂林
 *
 */
public class GovdocOrgHelper extends GovdocHelper {

	private static final Log LOGGER = LogFactory.getLog(GovdocOrgHelper.class);
	private static WorkflowApiManager wapi;

	public static WorkflowApiManager getWapi() {
		if (wapi == null) {
			try {
				wapi = (WorkflowApiManager) AppContext.getBean("wapi");
			} catch (Throwable e) {
				// log.error(e.getMessage(), e);
			}
		}
		return wapi;
	}

	/**
	 * 
	 * @param value
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("deprecation")
	public static Map<String, String> getUnitNameAndUnitId(String value) throws BusinessException {
		Map<String, String> result = new HashMap<String, String>();
		String unitName = value;
		StringBuilder unitId = new StringBuilder();
		Pattern pattern = Pattern.compile("([a-zA-Z]+[\\|])?([-]?[0-9]{10,20})");
		Matcher matcher = pattern.matcher(value);
		String group = null;
		Long id = null;
		while (matcher.find()) {
			try {
				group = matcher.group(0);
				id = Long.parseLong(matcher.group(2));
				if (group.startsWith("Account|")) {
					V3xOrgAccount account = orgManager.getAccountById(id);
					if (account != null) {
						unitName = unitName.replace(group, account.getName());
						unitId.append(group).append(",");
					}
				} else if (group.startsWith("OrgTeam|")) {
					EdocObjTeam team = govdocObjTeamManager.getObjTeamById(id);
					if (team != null) {
						unitName = unitName.replace(group, team.getName());
						unitId.append(group).append(",");
					}
				} else if (group.startsWith("Department|")) {
					V3xOrgDepartment department = orgManager.getDepartmentById(id);
					if (department != null) {
						unitName = unitName.replace(group, department.getName());
						unitId.append(group).append(",");
					}
				} else if (group.startsWith("ExchangeAccount|")) {
					ExchangeAccount account2 = govdocExchangeAccountManager.getExchangeAccount(id);
					if (account2 != null) {
						unitName = unitName.replace(group, account2.getName());
						unitId.append(group).append(",");
					}
				} else {
					V3xOrgEntity entity = orgManager.getEntityOnlyById(id);
					if (entity != null) {
						unitName = unitName.replace(group, entity.getName());
						unitId.append(group).append(",");
					}
				}
			} catch (Exception e) {
				LOGGER.error("getUnitNameAndUnitId：" + value);
			}
		}
		unitName = unitName.replaceAll("[,]+", ",");
		result.put("unitName", unitName);
		result.put("unitId", unitId.toString());
		return result;
	}

	/**
	 * 
	 * @param value
	 * @return
	 * @throws BusinessException
	 */
	public static Map<String, String> getUnitNameAndUnitId1(String value) throws BusinessException {
		Map<String, String> result = new HashMap<String, String>();
		if (value.contains("hiddenValue")) {
			value = value.replace("undefined", "");
			String[] str = value.split("hiddenValue");
			if (str.length == 1 || "null".equals(str[1])) {
				result.put("unitName", str[0]);
				result.put("unitId", "");
			} else if (str.length == 2) {
				result.put("unitName", str[0]);
				result.put("unitId", str[1]);
			}
		} else {
			return getUnitNameAndUnitId(value);
		}
		return result;
	}

	/**
	 * 解析相对角色并将其添加到集合
	 * 
	 * @param members
	 *            集合
	 * @param startMember
	 *            发起者
	 * @param memberRole
	 *            相对角色字符串
	 */
	@SuppressWarnings("rawtypes")
	public static void parseRoleMember(List<Map<String, Object>> members, Long startMember, String memberRole) {
		String startRole = "";
		String role = "";
		Long memberId = 0L;
		// 发起者
		if (memberRole.startsWith(WorkFlowConstants.ORGENT_META_KEY_SEDNER)) {
			startRole = memberRole.substring(0, WorkFlowConstants.ORGENT_META_KEY_SEDNER.length());
			role = memberRole.substring(WorkFlowConstants.ORGENT_META_KEY_SEDNER.length());
			memberId = startMember;
		} else if (memberRole.startsWith(WorkFlowConstants.ORGENT_META_KEY_NODEUSER)) {// 上节点
			startRole = memberRole.substring(0, WorkFlowConstants.ORGENT_META_KEY_NODEUSER.length());
			role = memberRole.substring(WorkFlowConstants.ORGENT_META_KEY_NODEUSER.length());
			memberId = AppContext.currentUserId();
		}
		try {
			V3xOrgMember tempMember = orgManager.getMemberById(memberId);
			Long accountId = tempMember.getOrgAccountId();
			Long departmentId = tempMember.getOrgDepartmentId();
			// 如果是上级部门 重新获取部门id
			if (role.startsWith(WorkFlowConstants.ORGENT_META_KEY_SUPERDEPT)) {
				role = role.substring(WorkFlowConstants.ORGENT_META_KEY_SUPERDEPT.length());
				String path = orgManager.getDepartmentById(departmentId).getPath();
				V3xOrgDepartment tempDepartment = orgManager.getDepartmentByPath(path.substring(0, path.length() - 4));
				if (tempDepartment != null) {
					departmentId = tempDepartment.getId();
				}
			}
			// 如果是发起者部门成员
			if (WorkFlowConstants.ORGENT_META_KEY_DEPMEMBER.equals(role)) {
				List<V3xOrgMember> memberList = orgManager.getMembersByDepartment(departmentId, true);
				for (V3xOrgMember v3xOrgMember : memberList) {
					Map<String, Object> memberMap = new HashMap<String, Object>();
					memberMap.put("id", v3xOrgMember.getId());
					memberMap.put("name", v3xOrgMember.getName());
					memberMap.put("orgAccountId", v3xOrgMember.getOrgAccountId());
					if (!members.contains(memberMap)) {
						members.add(memberMap);
					}
				}
				return;
			}
			if (Strings.isBlank(role) && Strings.isNotEmpty(startRole)) {
				if (WorkFlowConstants.ORGENT_META_KEY_SEDNER.equals(startRole)) {
					Map<String, Object> memberMap = new HashMap<String, Object>();
					memberMap.put("id", tempMember.getId());
					memberMap.put("name", tempMember.getName());
					memberMap.put("orgAccountId", tempMember.getOrgAccountId());
					if (!members.contains(memberMap)) {
						members.add(memberMap);
					}
				}
				return;
			}
			Map<String, Object> params = new HashMap<String, Object>();
			params.put("code", role);
			List<OrgRole> orgRoleList = orgDao.getAllRolePO(accountId, true, params, new FlipInfo());
			if (!orgRoleList.isEmpty()) {
				params.clear();
				params.put("id", orgRoleList.get(0).getId());
				List<V3xOrgMember> orgMembers = orgManager.getMembersByRole(departmentId, orgRoleList.get(0).getId());
				if (!orgMembers.isEmpty()) {
					String ids = "";
					for (Map map : members) {
						ids += map.get("id") + ",";
					}
					for (V3xOrgMember member : orgMembers) {
						// 如果已经有了,或者不是本部门的
						if (ids.contains(member.getId().toString())) {
							continue;
						}
						Map<String, Object> memberMap = new HashMap<String, Object>();
						memberMap.put("id", member.getId());
						memberMap.put("name", member.getName());
						memberMap.put("orgAccountId", member.getOrgAccountId());
						if (!members.contains(memberMap)) {
							members.add(memberMap);
						}
					}
				}
			}
		} catch (Exception e) {
			LOGGER.error(e);
		}
	}

	public static List<Long> getDeptIdList(Long currentAccountId) throws BusinessException {
		List<Long> deptIds = new ArrayList<Long>();
		deptIds.add(currentAccountId);
		deptIds.add(OrgConstants.GROUPID);
		List<V3xOrgDepartment> departmentIds = orgManager.getChildDepartments(Long.valueOf(currentAccountId), false);
		for (V3xOrgDepartment dept : departmentIds) {
			deptIds.add(dept.getId());
		}
		return deptIds;
	}

	public static List<Long> getUserDepIds(Long userId, Long currentAccountId, boolean isAdmin)
			throws BusinessException {
		StringBuilder depIdStr = new StringBuilder();

		List<Long> allUnitIdList = new ArrayList<Long>();
		allUnitIdList.add(OrgConstants.GROUPID);
		allUnitIdList.add(currentAccountId);
		depIdStr.append(OrgConstants.GROUPID);
		depIdStr.append("," + currentAccountId);

		try {
			Long currentDeptId = orgManager.getCurrentDepartment().getId();
			allUnitIdList.add(currentDeptId);
			depIdStr.append("," + currentDeptId);

			List<V3xOrgDepartment> parentList = orgManager.getAllParentDepartments(currentDeptId);
			if (Strings.isNotEmpty(parentList)) {
				for (V3xOrgDepartment bean : parentList) {
					if (!allUnitIdList.contains(bean.getId())) {
						allUnitIdList.add(bean.getId());
						depIdStr.append("," + bean.getId());
					}
				}
			}
		} catch (Exception e) {
		}

		List<MemberPost> postList = orgManager.getMemberPosts(currentAccountId, AppContext.currentUserId());
		for (MemberPost post : postList) {
			if (post.getDepId() != null && !allUnitIdList.contains(post.getDepId())) {
				allUnitIdList.add(post.getDepId());
				depIdStr.append("," + post.getDepId());
			}
		}
		if (isAdmin) {
			List<V3xOrgDepartment> dList = orgManager.getChildDepartments(currentAccountId, true);
			for (V3xOrgDepartment subDep : dList) {
				if (!allUnitIdList.contains(subDep.getId())) {
					allUnitIdList.add(subDep.getId());
					depIdStr.append(",").append(subDep.getId());
				}
			}
		}
		return allUnitIdList;
	}

	public static String getMarkAclEntityName(List<EdocMarkAcl> aclList, Long markDefId) throws BusinessException {
		String entityName = "";
		List<V3xOrgEntity> aclEntity = new ArrayList<V3xOrgEntity>();
		// 客开 项目名称： [修改功能：如果不是本单位加上区隔] 作者：fzc 修改日期：2018-5-10 start
		Long currentUserAccountId = AppContext.getCurrentUser().getAccountId();
		V3xOrgEntity orgEntity = null;
		for (EdocMarkAcl markAcl : aclList) {
			if (markAcl.getMarkDefId().longValue() == markDefId.longValue()) {
				orgEntity = orgManager.getEntity(markAcl.getAclType(), markAcl.getDeptId());
				if (orgEntity != null && currentUserAccountId.equals(orgEntity.getOrgAccountId())) {
					entityName += orgEntity.getName() + "、";
				} else if (orgEntity != null) {
					V3xOrgAccount temp = orgManager.getAccountById(orgEntity.getOrgAccountId());
					entityName += orgEntity.getName() + "(" + temp.getShortName() + ")" + "、";
				}
				// 客开 项目名称： [修改功能：如果不是本单位加上区隔] 作者：fzc 修改日期：2018-5-10 end
				aclEntity.add(orgEntity);
			}
		}
		if (Strings.isNotBlank(entityName)) {
			entityName = entityName.substring(0, entityName.length() - 1);
		}
		return entityName;
	}

	public static V3xOrgEntity getOrgEntity(String entityType, Long entityId) throws BusinessException {
		return orgManager.getEntity(entityType, entityId);
	}

	public static V3xOrgMember getMemberById(Long memberId) {
		try {
			V3xOrgMember member = orgManager.getMemberById(memberId);
			return member;
		} catch (Exception e) {

		}
		return null;
	}

	public static String getMemberNameById(Long memberId) {
		try {
			V3xOrgMember member = orgManager.getMemberById(memberId);
			if (member != null) {
				return member.getName();
			}
		} catch (Exception e) {

		}
		return null;
	}

	/*
	 * 获取人员id
	 */
	public static List<V3xOrgMember> getMembersByTypeAndIds(String typeAndIds) {
		Set<V3xOrgMember> members = new HashSet<V3xOrgMember>();
		try {
			members = orgManager.getMembersByTypeAndIds(typeAndIds);
		} catch (BusinessException e) {
			LOGGER.error(e);
		}
		List<V3xOrgMember> memberList = new ArrayList<V3xOrgMember>(members);
		return memberList;
	}

	/**
	 * 获取组织结构等对象
	 * 
	 * @param memberId
	 * @param entityClass
	 * @return
	 */
	public static <T extends V3xOrgEntity> T getEntityById(Class<T> classType, Long id) throws BusinessException {
		return orgManager.getEntityById(classType, id);
	}

	/**
	 * 获取多维组织
	 * 
	 * @param input
	 * @param primaryInput
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("deprecation")
	public static String getMutiDepAndAccount(String input, String primaryInput) throws BusinessException {
		StringBuilder hiddenValue = new StringBuilder();
		if (Strings.isNotBlank(primaryInput)) {
			String[] oldData = primaryInput.split(",");
			for (int i = 0; i < oldData.length; i++) {
				String temp = oldData[i];
				if (temp.indexOf("|") != -1) {
					String orgType = temp.substring(0, temp.indexOf("|"));
					Long orgId = -1L;
					try {
						orgId = Long.valueOf(temp.substring(temp.indexOf("|") + 1));
					} catch (Exception e) {
						continue;
					}
					String name = "";
					if ("Account".equals(orgType) || "Department".equals(orgType)) {
						V3xOrgEntity entity = orgManager.getEntityOnlyById(orgId);
						name = entity.getName();
					} else if ("OrgTeam".equals(orgType)) {
						name = OrgHelper.showOrgEntities(orgId + "", "OrgTeam", "|");
					} else if ("ExchangeAccount".equals(orgType)) {
						name = OrgHelper.showOrgEntities(orgId + "", "ExchangeAccount", "|");
					}
					if (Strings.isBlank(name)) {
						continue;
					}
					int nameIndex = input.indexOf(name);
					if (nameIndex > -1) {
						hiddenValue.append(temp);
						if (i < oldData.length - 1) {
							hiddenValue.append(",");
						}
					}
				}

			}
		}
		return hiddenValue.toString();
	}

	/**
	 * 
	 * @param value
	 * @return
	 * @throws BusinessException
	 */
	public static List<GovdocWorkflowVO> transGovdocWorkflowVO(String value) throws BusinessException {
		List<GovdocWorkflowVO> result = new ArrayList<GovdocWorkflowVO>();
		String[] orgStr = value.split(",");
		String[] orgInfo = null;
		for (String org : orgStr) {
			orgInfo = org.split("\\|");
			try {
				String group = orgInfo[0];
				if (group.startsWith("Account")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgAccount account = orgManager.getAccountById(id);
					if (account != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(account.getId());
						vo.setAccountId(account.getOrgAccountId());
						vo.setName(account.getName());
						vo.setType("Account");
						vo.setM3Img("account.png");
						result.add(vo);
					}
				} else if (group.startsWith("OrgTeam")) {
					Long id = Long.parseLong(orgInfo[1]);
					EdocObjTeam team = govdocObjTeamManager.getObjTeamById(id);
					if (team != null) {
					}
				} else if (group.startsWith("Department")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgDepartment department = orgManager.getDepartmentById(id);
					if (department != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(department.getId());
						vo.setAccountId(department.getOrgAccountId());
						vo.setName(department.getName());
						vo.setType("Department");
						vo.setM3Img("department.png");
						result.add(vo);
					}
				} else if (group.startsWith("ExchangeAccount")) {
					Long id = Long.parseLong(orgInfo[1]);
					ExchangeAccount account2 = govdocExchangeAccountManager.getExchangeAccount(id);
					if (account2 != null) {
					}
				} else if (group.startsWith("Member")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgMember entity = orgManager.getMemberById(id);
					if (entity != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(entity.getId());
						vo.setAccountId(entity.getOrgAccountId());
						vo.setName(entity.getName());
						vo.setType("Member");
						vo.setDepName(orgManager.getDepartmentById(entity.getOrgDepartmentId()).getName());
						result.add(vo);
					}
				} else if (group.startsWith("Node")) {
					String roleName = RoleTransferUtil.getRoleName(orgInfo[1]);
					GovdocWorkflowVO vo = new GovdocWorkflowVO();
					vo.setId(orgInfo[1]);
					if (roleName == null) {
						roleName = getWapi().getRoleShowNameByName(orgInfo[1],
								AppContext.getCurrentUser().getAccountId());
						if (Strings.isBlank(roleName)) {
							Pattern p = Pattern.compile("[-]?[0-9]+");
							Matcher m = p.matcher(orgInfo[1]);
							if (m.find()) {
								// 通过数据库查询
								Long id = Long.parseLong(m.group());
								V3xOrgRole role = orgManager.getRoleById(id);
								String privRoleName = orgInfo[1].substring(0, orgInfo[1].indexOf(id + ""));
								vo.setName(RoleTransferUtil.getRoleName(privRoleName)
										+ ResourceUtil.getString("sys.role.rolename." + role.getCode()));
							}
						} else {
							vo.setName(roleName);
						}
					} else {
						vo.setName(roleName);
					}
					vo.setType("Node");
					vo.setM3Img("department.png");
					result.add(vo);
				} else if (group.startsWith("Team")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgTeam entity = orgManager.getTeamById(id);
					if (entity != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(entity.getId());
						vo.setAccountId(entity.getOrgAccountId());
						vo.setName(entity.getName());
						vo.setType("Team");
						vo.setM3Img("group.png");
						result.add(vo);
					}
				} else if (group.startsWith("Post")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgPost entity = orgManager.getPostById(id);
					if (entity != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(entity.getId());
						vo.setAccountId(entity.getOrgAccountId());
						vo.setName(entity.getName());
						vo.setType("Post");
						vo.setM3Img("post.png");
						result.add(vo);
					}
				} else if (group.startsWith("Level")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgLevel entity = orgManager.getLevelById(id);
					if (entity != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(entity.getId());
						vo.setAccountId(entity.getOrgAccountId());
						vo.setName(entity.getName());
						vo.setType("Level");
						vo.setM3Img("level.png");
						result.add(vo);
					}
				} else if (group.startsWith("BusinessDepartment")) {
					Long id = Long.parseLong(orgInfo[1]);
					V3xOrgEntity entity = orgManager.getEntity("BusinessDepartment", id);
					if (entity != null) {
						GovdocWorkflowVO vo = new GovdocWorkflowVO();
						vo.setId(entity.getId());
						vo.setAccountId(entity.getOrgAccountId());
						vo.setName(entity.getPreName() + entity.getName());
						vo.setType("BusinessDepartment");
						vo.setM3Img("department.png");
						result.add(vo);
					}
				}
			} catch (Exception e) {
				LOGGER.error("transGovdocWorkflowVO：" + value);
			}
		}
		return result;
	}

}
