package com.seeyon.apps.govdoc.helper;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.collaboration.vo.SeeyonPolicy;
import com.seeyon.apps.govdoc.constant.GovdocEnum.GovdocWorkflowTypeEnum;
import com.seeyon.apps.govdoc.listener.GovdocWorkflowEventListener;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.GlobalNames;
import com.seeyon.ctp.common.ModuleType;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.content.ContentConfig;
import com.seeyon.ctp.common.content.ContentViewRet;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.content.mainbody.CtpContentAllBean;
import com.seeyon.ctp.common.content.mainbody.MainbodyStatus;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.template.CtpTemplate;
import com.seeyon.ctp.form.bean.FormBean;
import com.seeyon.ctp.form.bean.FormDataMasterBean;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.ctp.workflow.dao.IProcessTemplateDao;
import com.seeyon.ctp.workflow.engine.enums.BPMSeeyonPolicySetting;
import com.seeyon.ctp.workflow.exception.BPMException;
import com.seeyon.ctp.workflow.manager.ProcessTemplateManager;
import com.seeyon.ctp.workflow.po.ProcessTemplete;
import com.seeyon.ctp.workflow.util.WorkflowUtil;
import com.seeyon.ctp.workflow.vo.CPMatchResultVO;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.v3x.edoc.domain.EdocSummary;

import net.joinwork.bpm.definition.BPMAbstractNode;
import net.joinwork.bpm.definition.BPMActivity;
import net.joinwork.bpm.definition.BPMAndRouter;
import net.joinwork.bpm.definition.BPMEnd;
import net.joinwork.bpm.definition.BPMHumenActivity;
import net.joinwork.bpm.definition.BPMProcess;
import net.joinwork.bpm.definition.BPMSeeyonPolicy;
import net.joinwork.bpm.definition.BPMStart;
import net.joinwork.bpm.definition.BPMTransition;
import net.joinwork.bpm.engine.wapi.WorkflowBpmContext;	

/**
 * 公文流程辅助类
 * @author 唐桂林
 *
 */
public class GovdocWorkflowHelper extends GovdocHelper {

	private static final Log LOGGER = LogFactory.getLog(GovdocWorkflowHelper.class);	
    
	/**
     * 内容查看相关处理，返回内容查看列表（支持多正文）
     * @param moduleType 模块类型，默认值ModuleType.collaboration.getKey()
     * @param moduleId 模块ID，默认值-1（新建），否则为模块多内容新建
     * @param rightId 新建内容权限ID，默认值-1，例如表单模板权限ID
     * @return 内容查看Content对象列表（支持多正文）
     * @throws BusinessException 内容查看相关异常
     */
    @SuppressWarnings({ "deprecation", "unchecked" })
	public static ContentViewRet contentView(ModuleType moduleType, Long moduleId, Long affairId, int viewState,
            String rightId) throws BusinessException {
        if (moduleType == null){
        	moduleType = ModuleType.collaboration;
        }           
        if (moduleId == null){
        	moduleId = -1L;
        }           
        HttpServletRequest request = (HttpServletRequest) AppContext
                .getThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);
        List<CtpContentAllBean> contentList = ctpMainbodyManager.transContentViewResponse(moduleType, moduleId, viewState,
                rightId);
        if(contentList.size() == 0){//流程模板查看增加个空正文返回
        		CtpContentAllBean content_null = new CtpContentAllBean();
        		content_null.setViewState(viewState);
        		content_null.setStatus(MainbodyStatus.STATUS_RESPONSE_VIEW);
        		content_null.setRightId(rightId);
        		content_null.setContentType(10);
                contentList.add(content_null);
        }
        
        request.setAttribute("contentList", contentList);
		
        ContentViewRet context = new ContentViewRet();
        context.setModuleId(moduleId);
        context.setModuleType(moduleType.getKey());
        context.setAffairId(affairId);
        ContentConfig contentCfg = ContentConfig.getConfig(moduleType);

        if (contentCfg.getCommentTemplate() != null) {
            Map<String,Object> commentsMap = ctpCommentManager.getCommentsWithForward(moduleType,moduleId);
            Comment commentDraft = null;

            List<Comment> draftList = (List<Comment>) commentsMap.get("commentDraftList");
            if(Strings.isNotEmpty(draftList)){
                for (Comment tmpDraft : draftList) {
                    //获取当前用户评论草稿
                    if (AppContext.currentUserId() == tmpDraft.getCreateId()
                            && (affairId == null || (affairId != null && affairId.equals(tmpDraft.getAffairId())))) {
                        commentDraft = tmpDraft;
                        commentDraft.setId(UUIDLong.longUUID());
                        break;
                    }
                }
            }

            request.setAttribute("commentDraft", commentDraft);
            
            List<StateEnum> states = new ArrayList<StateEnum>();
            states.add(StateEnum.col_sent);
            states.add(StateEnum.col_done);
            states.add(StateEnum.col_pending);
            states.add(StateEnum.col_waitSend);
            
            List<CtpAffair> pushMessageList = affairManager.getAffairs(moduleId, states);
            //排序顺序规则，发起人、已办、暂存待办
            Collections.sort(pushMessageList, new Comparator<CtpAffair>() {
                @Override
                public int compare(CtpAffair o1, CtpAffair o2) {
                    if (o1.getState() == StateEnum.col_sent.getKey()){
                    	return -1;
                    }                       
                    else if (o2.getState() == StateEnum.col_sent.getKey()){
                    	return 1;
                    }else {
                        if (o1.getState() == StateEnum.col_done.key()){
                        	return -1;
                        }else if (o2.getState() == StateEnum.col_done.key()){
                        	return 1;
                        }else{
                        	return 0;
                        }                            
                    }
                }
            });
            //过滤掉自己和重复项
            List<Long> memberIdList = new ArrayList<Long>();
            //消息推送
            List<CtpAffair> pushMessageListAffair = new ArrayList<CtpAffair>();
            for (CtpAffair r : pushMessageList) {
                //只显示已发、暂存待办和、已办的、回退者
                if ((r.getSubState() == SubStateEnum.col_pending_ZCDB.key() && r.getState() == StateEnum.col_pending
                        .key()) || r.getState() == StateEnum.col_done.key() || r.getState() == StateEnum.col_sent.key()
                        || r.getSubState() == SubStateEnum.col_pending_specialBack.key()
                        || r.getSubState() == SubStateEnum.col_pending_specialBacked.key()
                        || r.getSubState() == SubStateEnum.col_pending_specialBackCenter.key()) {
                    if (!r.getMemberId().equals(AppContext.currentUserId()) && !memberIdList.contains(r.getMemberId())) {
                        memberIdList.add(r.getMemberId());
                        pushMessageListAffair.add(r);
                    }
                }
            }
            request.setAttribute("commentPushMessageToMembersList", pushMessageListAffair);

            context.setCommentMaxPath((String) AppContext.getRequestContext("commentMaxPathStr"));
        }
        request.setAttribute("__huanhang", "\r\n");
        request.setAttribute("contentContext", context);
        request.setAttribute("contentCfg", contentCfg);
        return context;
    }
    
    /**
	 * 取当前节点的父人工节点
	 * @param activity   当前人工节点
	 * @return
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static List getParent(BPMActivity activity){
		List humenList = new ArrayList();
        List<BPMTransition> transitions = activity.getUpTransitions();
        for (BPMTransition trans : transitions) {
            BPMAbstractNode parent = trans.getFrom();
            BPMSeeyonPolicy policy= parent.getSeeyonPolicy();
        	String policyId= policy.getId();
        	String isDelete= policy.getIsDelete();
        	if("false".equals(isDelete)){
        		if (parent.getNodeType() == BPMAbstractNode.NodeType.humen) {
                	//处理空节点及知会节点
                	if("zhihui".equals(policyId) || "inform".equals(policyId)){
                		humenList.addAll(getParent((BPMActivity) parent));
                	}else{
                		humenList.add((BPMHumenActivity) parent);
                	}
                } else if(parent.getNodeType() == BPMAbstractNode.NodeType.start){
                	humenList.add(parent);
                } else if (parent.getNodeType() == BPMAbstractNode.NodeType.join || parent.getNodeType() == BPMAbstractNode.NodeType.split) {
                    humenList.addAll(getParent((BPMActivity) parent));
                }
        	}
        }
        return humenList;
	}
    
    /**
	 * 取当前节点的父人工节点
	 * @param activity   当前人工节点
	 * @return
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static List getNext(BPMActivity activity) {
		List humenList = new ArrayList();
		List<BPMTransition> transitions = activity.getDownTransitions();
        for (BPMTransition trans : transitions) {
            BPMAbstractNode next = trans.getTo();
            BPMSeeyonPolicy policy= next.getSeeyonPolicy();
        	String policyId= policy.getId();
        	String isDelete= policy.getIsDelete();
        	if("false".equals(isDelete)){
        		if (next.getNodeType() == BPMAbstractNode.NodeType.humen) {
                	//处理空节点及知会节点
                	if("zhihui".equals(policyId) || "inform".equals(policyId)){
                		humenList.addAll(getNext((BPMActivity) next));
                	}else{
                		humenList.add((BPMHumenActivity) next);
                	}
                } else if(next.getNodeType() == BPMAbstractNode.NodeType.start){
                	humenList.add(next);
                } else if (next.getNodeType() == BPMAbstractNode.NodeType.join || next.getNodeType() == BPMAbstractNode.NodeType.split) {
                    humenList.addAll(getNext((BPMActivity) next));
                }
        	}
        }
        return humenList;
	}
	
	/**
	 * 验证流程节点权限节点权限
	 * @param affair
	 * @param pageNodePolicy
	 * @return
	 * @throws BusinessException
	 */
	public static String checkNodePolicyChange(CtpAffair affair, String pageNodePolicy) throws BusinessException {
		String errorMsg = "";
		if(affair == null
        		|| affair.getState().equals(StateEnum.col_waitSend.getKey())
        		|| affair.getState().equals(StateEnum.col_sent.getKey())){
			return errorMsg;
		}
		SeeyonPolicy affairNodePolicy = collaborationApi.getPolicyByAffair(affair);
		if(Strings.isNotBlank(pageNodePolicy) && !affairNodePolicy.getId().equals(pageNodePolicy)){
			errorMsg = ResourceUtil.getString("collaboration.nodePolicy.change");
		}
		return errorMsg;
	}
	
	/**
	 * 获取流程的下一节点人员
	 * @param formBean
	 * @param dataMasterBean
	 * @param user
	 * @param ctpTemplate
	 * @return
	 * @throws BPMException
	 */
	@SuppressWarnings("unchecked")
	public static List<V3xOrgMember> getPersonNextNode(FormBean formBean, FormDataMasterBean dataMasterBean, CtpTemplate ctpTemplate, User user, boolean isAddCache) throws BPMException{
		List<V3xOrgMember> listmember = new ArrayList<V3xOrgMember>();
		WorkflowBpmContext context = new WorkflowBpmContext();
		context.setAppName(ModuleType.edoc.name());
		context.getBusinessData().put("subAppName", GovdocWorkflowTypeEnum.formedoc.name());
        context.setDebugMode(false);
        context.setFormData(""+formBean.getId());
        context.setFormAppId(""+formBean.getId());
        context.setMastrid(""+dataMasterBean.getId());
        //设置正文内容，用来发送邮件的时候显示正文内容
        context.setStartUserId(String.valueOf(user.getId()));
        context.setStartUserName(user.getName());
        context.setBusinessData(GovdocWorkflowEventListener.CURRENT_OPERATE_MEMBER_ID, user.getId());
        context.setStartAccountId(String.valueOf(user.getLoginAccount()));
        context.setStartAccountName("seeyon");
        context.setVersion("2.0");
        
        WorkflowApiManager wapi = (WorkflowApiManager)AppContext.getBean("wapi");
        if(isAddCache) {
        	wapi.addFormDataDisplayName(context, null);
        }
        String xml1 = wapi.selectWrokFlowTemplateXml(ctpTemplate.getWorkflowId().toString());
        BPMProcess bpmProcess = BPMProcess.fromXML(xml1);
        context.setProcess(bpmProcess);
        List<BPMTransition> bpmTransitions = bpmProcess.getLinks();
        for (BPMTransition bpmTransition : bpmTransitions) {
	       	 BPMAbstractNode node = bpmTransition.getFrom();
	       	 if(node instanceof BPMStart){//如果是发起节点
	       		BPMAbstractNode nextNode = bpmTransition.getTo();
	       		if("split".equals(nextNode.getName())){
	        		List<BPMTransition> bpmTransitions2 = nextNode.getDownTransitions();
	        		for (BPMTransition bpmTransitio : bpmTransitions2) {
			       		try {
								listmember.addAll(wapi.getUserList("", (BPMHumenActivity)bpmTransitio.getTo(),context,true));
							} catch (Exception e) {
								LOGGER.error(e);
							}
			       	}
	        	}else{
		       		if(nextNode instanceof BPMHumenActivity || nextNode instanceof BPMEnd){//如果当前节点不是起始节点
		       			try {
								listmember.addAll(wapi.getUserList("", (BPMHumenActivity)nextNode,context,true));
							} catch (Exception e) {
								LOGGER.error(e);
								//e.printStackTrace();
							}
		       		}
	        	}
	        }
		}
        return listmember;
	}
	
	@SuppressWarnings("unchecked")
	public static boolean canMergeDealByType(BPMSeeyonPolicySetting.MergeDealType dealType,EdocSummary summary){
		String mergeDealType = "";
		if(summary!=null){
			mergeDealType = summary.getMergeDealType();
		}
		if(Strings.isNotBlank(mergeDealType)){
			String mergeDealTypeValue= dealType.getValue();
			Map<String,String> mergeDealTypeMap  = (Map<String, String>) JSONUtil.parseJSONString(mergeDealType);
			if(null!=mergeDealTypeMap && !mergeDealTypeMap.isEmpty()){
				return mergeDealTypeValue.equals(mergeDealTypeMap.get(dealType.name()));
			}
		}
		return false;
	}
	public static CPMatchResultVO isSelectPersonCondition(CtpTemplate template, long sender,long masterId,long loginAccountId) throws BPMException {
        //选分支,选人的判断,isPop 为true则保存待发
        WorkflowBpmContext wfContext = new WorkflowBpmContext();
        wfContext.setProcessId(null);
        wfContext.setCaseId(-1L);
        wfContext.setCurrentActivityId(null);
        wfContext.setCurrentWorkitemId(-1L);
        wfContext.setMastrid(String.valueOf(masterId));
        wfContext.setFormData(String.valueOf(masterId));
        wfContext.setFormAppId(String.valueOf(masterId));
        wfContext.setStartUserId(String.valueOf(sender));
        wfContext.setCurrentUserId(String.valueOf(sender));
        wfContext.setProcessTemplateId(String.valueOf(template.getWorkflowId()));
        wfContext.setAppName(ModuleType.edoc.name());
//      wfContext.setStartAccountId(String.valueOf(member.getOrgAccountId()));
//      wfContext.setCurrentAccountId(String.valueOf(member.getOrgAccountId()));
        wfContext.setStartAccountId(String.valueOf(loginAccountId));
        wfContext.setCurrentAccountId(String.valueOf(loginAccountId));
        WorkflowApiManager wapi=(WorkflowApiManager) AppContext.getBean("wapi");
        CPMatchResultVO result = wapi.transBeforeInvokeWorkFlow(wfContext, new CPMatchResultVO());
		if (result.getIsPop()) {
			// 防护 GOVB-3268 7.1签收流程是单人执行
			try {
				ProcessTemplateManager processTemplateManager = (ProcessTemplateManager) AppContext
						.getBean("processTemplateManager");
				String processXml = processTemplateManager
						.selectProcessTempateXml(String.valueOf(template.getWorkflowId()));
				if (Strings.isNotBlank(processXml)) {
					BPMProcess process = BPMProcess.fromXML(processXml);
					BPMTransition tran = (BPMTransition) process.getStart().getDownTransitions().get(0);
					BPMAbstractNode node = tran.getTo();
					if (node instanceof BPMHumenActivity) {// 单节点
						BPMHumenActivity activity = (BPMHumenActivity) node;
						if (activity.isSingleProcessMode()) {
							activity.getSeeyonPolicy().setProcessMode("competition");
							// 修改流程图
							IProcessTemplateDao processTemplateDao = (IProcessTemplateDao) AppContext
									.getBean("IProcessTemplateDao");
							ProcessTemplete pt = processTemplateDao
									.selectProcessTempate(String.valueOf(template.getWorkflowId()));
							pt.setWorkflow(process.toXML());
							processTemplateDao.updateProcessTemplate(pt);
							//再校验一次是为了确认已设置为竞争模式后是否还有其他问题
							result = wapi.transBeforeInvokeWorkFlow(wfContext, new CPMatchResultVO());
						}
					} else if (node instanceof BPMAndRouter) {// 并发多节点
						LOGGER.error("交换流程配置有问题,loginAccountId:" + loginAccountId + " template:" + template.getId());
					}
				}

			} catch (Exception e) {
				LOGGER.error("交换流程重置竞争模式失败", e);
			}

		}
        return result;
    }
}
